@@ -294,7 +294,7 @@ posteriorClosed<-function(parms,DM,mms,priorparms,gq){
   }
 }
 
-checkClosed<-function(parms,parmlist,mms,DM,iter,bin,thin,burnin,taccept,tuneadjust,npoints,maxnumbasis,a0delta,a0alpha,b0alpha,a,sigma2_mu0){
+checkClosed<-function(parms,parmlist,mms,DM,iter,adapt,bin,thin,burnin,taccept,tuneadjust,npoints,maxnumbasis,a0delta,a0alpha,b0alpha,a,sigma2_mu0){
   
   if(mms@data.type!="sometimes" & any(parms=="alpha")) stop("Parameter 'alpha' only applies to models for the 'sometimes' data type")
   
@@ -309,12 +309,13 @@ checkClosed<-function(parms,parmlist,mms,DM,iter,bin,thin,burnin,taccept,tuneadj
     if(!all(match(params,parmlist,nomatch=0))) stop(paste0("'",params[match(params,parmlist,nomatch=0)==0],"' is not a valid parameter"))
   }  
   
-  if((bin<1 | bin>iter) & iter>0) stop(paste("'bin' must be >0 and <",iter))
-  if(thin>max(1,floor((iter-burnin)/2)) | thin<1) stop(paste("'thin' must be >0 and <=",max(1,floor((iter-burnin)/2))))
+  if(adapt<0) stop("'adapt' must be >=0")
+  if((bin<1 | bin>iter) & iter>0) stop("'bin' must be >0 and <",iter)
+  if(thin>max(1,floor((iter-burnin)/2)) | thin<1) stop("'thin' must be >0 and <=",max(1,floor((iter-burnin)/2)))
   if(taccept<=0 | taccept>1) stop ("'taccept' must be >0 and <=1")
   if(tuneadjust<=0 | tuneadjust>1) stop ("'tuneadjust' must be >0 and <=1")
   if(npoints<1) stop("'npoints' must be greater than 0")
-  if(maxnumbasis<1 | maxnumbasis>mms@ncolbasis) stop(paste("'maxnumbasis' must be between 1 and ",mms@ncolbasis))
+  if(maxnumbasis<1 | maxnumbasis>mms@ncolbasis) stop("'maxnumbasis' must be between 1 and ",mms@ncolbasis)
   if(!all(c(a0delta,a0alpha,b0alpha,a,sigma2_mu0)>0)) stop("'a0delta', 'a0alpha', 'b0alpha', 'a', and 'sigma2_mu0' must be >0")
   
   mod.p.h<-DM$mod.p.h
@@ -429,31 +430,31 @@ processClosedchains<-function(chains,params,DM,M,noccas,nchains,iter,burnin,thin
 #'
 #' @param covs A data frame of temporal covariates for detection probabilities (ignored unless \code{mms=NULL}). The number of rows in the data frame must equal the number of sampling occasions. Covariate names cannot be "time", "age", or "h"; these names are reserved for temporal, behavioral, and individual effects when specifying \code{mod.p} and \code{mod.phi}.
 #' @param mms An optional object of class \code{multimarksetup-class}; if \code{NULL} it is created. See \code{\link{processdata}}.
-#' @param mod.p Model formula for detection probability. For example, \code{mod.p=~1} specifies no effects (i.e., intercept only), \code{mod.p~time} specifies temporal effects, \code{mod.p~c} specifies behavioral reponse (i.e., trap "happy" or "shy"), \code{mod.p~h} specifies individual heterogeneity, and \code{mod.p~time+c} specifies additive temporal and behavioral effects .
-#' @param mod.delta Model formula for conditional probabilities of type 1 (delta_1) and type 2 (delta_2) encounters, given detection. Currently only \code{mod.delta=~1} (i.e., \eqn{delta_1 = delta_2}) and \code{mod.delta=~type} (i.e., \eqn{delta_1 \ne delta_2}) are implemented.
-#' @param parms A character vector giving the names of the parameters and latent variables to monitor. Possible parameters are logit-scale detection probability parameters ("\code{pbeta}"), population abundance ("\code{N}"), conditional probability of type 1 or type 2 encounter, given detection ("\code{delta})", probability of simultaneous type 1 and type 2 detection, given both types encountered ("\code{alpha}"), logit-scale individual heterogeneity variance term ("\code{sigma2_zp}"), logit-scale individual effects ("\code{zp}"), and the probability that any given observed history corresponds to a legitmate individual ("\code{psi}"). Individual encounter history indices ("\code{H}") and the log posterior density ("\code{logPosterior}") may also be monitored. Setting \code{parms="all"} monitors all possible parameters and latent variables .
-#' @param nchains The number of parallel MCMC chains for the model .
-#' @param iter The number of MCMC iterations .
-#' @param adapt The number of iterations for proposal distribution adaptation. If \code{adapt = 0} then no adaptation occurs .
-#' @param bin Bin length for calculating acceptance rates during adaptive phase (\code{0 < bin <= iter}) .
-#' @param thin Thinning interval for monitored parameters (\code{0 < thin <= max(1,floor((iter-burnin)/2))}) .
-#' @param burnin Number of burn-in iterations (\code{0 <= burnin < iter}) .
-#' @param taccept Target acceptance rate during adaptive phase (\code{0 < taccept <= 1}). Acceptance rate is monitored every \code{bin} iterations. Default is \code{taccept = 0.44} .
-#' @param tuneadjust Adjustment term during adaptive phase (\code{0 < tuneadjust <= 1}). If acceptance rate is less than \code{taccept}, then proposal term (\code{proppbeta}, \code{propzp}, or \code{propsigmap}) is multiplied by \code{tuneadjust}. If acceptance rate is greater than or equal to \code{taccept}, then proposal term is divided by \code{tuneadjust}. Default is \code{tuneadjust = 0.95} .
-#' @param proppbeta Scaler or vector (of length k) specifying the initial standard deviation of the Normal(pbeta[j], proppbeta[j]) proposal distribution. If \code{proppbeta} is a scaler, then this value is used for all j = 1, ..., k. Default is \code{proppbeta = 0.1} .
-#' @param propzp Scaler or vector (of length M) specifying the initial standard deviation of the Normal(zp[i], propzp[i]) proposal distribution. If \code{propzp} is a scaler, then this value is used for all i = 1, ..., M individuals. Default is \code{propzp = 1} .
-#' @param propsigmap Scaler specifying the initial Gamma(shape = 1/\code{propsigmap}, scale = sigma_zp*\code{propsigmap}) proposal distribution for sigma_zp = sqrt(sigma2_zp). Default is \code{propsigmap=1} .
-#' @param npoints Number of Gauss-Hermite quadrature points to use for numerical integration. Accuracy increases with number of points, but so does computation time .
-#' @param maxnumbasis Maximum number of basis vectors to use when proposing latent history frequency updates. Default is \code{maxnumbasis = 1}, but higher values can potentially improve mixing .
+#' @param mod.p Model formula for detection probability. For example, \code{mod.p=~1} specifies no effects (i.e., intercept only), \code{mod.p~time} specifies temporal effects, \code{mod.p~c} specifies behavioral reponse (i.e., trap "happy" or "shy"), \code{mod.p~h} specifies individual heterogeneity, and \code{mod.p~time+c} specifies additive temporal and behavioral effects.
+#' @param mod.delta Model formula for conditional probabilities of type 1 (delta_1) and type 2 (delta_2) encounters, given detection. Currently only \code{mod.delta=~1} (i.e., \eqn{\delta_1 = \delta_2}) and \code{mod.delta=~type} (i.e., \eqn{\delta_1 \ne \delta_2}) are implemented.
+#' @param parms A character vector giving the names of the parameters and latent variables to monitor. Possible parameters are logit-scale detection probability parameters ("\code{pbeta}"), population abundance ("\code{N}"), conditional probability of type 1 or type 2 encounter, given detection ("\code{delta})", probability of simultaneous type 1 and type 2 detection, given both types encountered ("\code{alpha}"), logit-scale individual heterogeneity variance term ("\code{sigma2_zp}"), logit-scale individual effects ("\code{zp}"), and the probability that any given observed history corresponds to a legitmate individual ("\code{psi}"). Individual encounter history indices ("\code{H}") and the log posterior density ("\code{logPosterior}") may also be monitored. Setting \code{parms="all"} monitors all possible parameters and latent variables.
+#' @param nchains The number of parallel MCMC chains for the model.
+#' @param iter The number of MCMC iterations.
+#' @param adapt The number of iterations for proposal distribution adaptation. If \code{adapt = 0} then no adaptation occurs.
+#' @param bin Bin length for calculating acceptance rates during adaptive phase (\code{0 < bin <= iter}).
+#' @param thin Thinning interval for monitored parameters.
+#' @param burnin Number of burn-in iterations (\code{0 <= burnin < iter}).
+#' @param taccept Target acceptance rate during adaptive phase (\code{0 < taccept <= 1}). Acceptance rate is monitored every \code{bin} iterations. Default is \code{taccept = 0.44}.
+#' @param tuneadjust Adjustment term during adaptive phase (\code{0 < tuneadjust <= 1}). If acceptance rate is less than \code{taccept}, then proposal term (\code{proppbeta}, \code{propzp}, or \code{propsigmap}) is multiplied by \code{tuneadjust}. If acceptance rate is greater than or equal to \code{taccept}, then proposal term is divided by \code{tuneadjust}. Default is \code{tuneadjust = 0.95}.
+#' @param proppbeta Scaler or vector (of length k) specifying the initial standard deviation of the Normal(pbeta[j], proppbeta[j]) proposal distribution. If \code{proppbeta} is a scaler, then this value is used for all j = 1, ..., k. Default is \code{proppbeta = 0.1}.
+#' @param propzp Scaler or vector (of length M) specifying the initial standard deviation of the Normal(zp[i], propzp[i]) proposal distribution. If \code{propzp} is a scaler, then this value is used for all i = 1, ..., M individuals. Default is \code{propzp = 1}.
+#' @param propsigmap Scaler specifying the initial Gamma(shape = 1/\code{propsigmap}, scale = sigma_zp * \code{propsigmap}) proposal distribution for sigma_zp = sqrt(sigma2_zp). Default is \code{propsigmap=1}.
+#' @param npoints Number of Gauss-Hermite quadrature points to use for numerical integration. Accuracy increases with number of points, but so does computation time.
+#' @param maxnumbasis Maximum number of basis vectors to use when proposing latent history frequency updates. Default is \code{maxnumbasis = 1}, but higher values can potentially improve mixing.
 #' @param a0delta Scaler or vector (of length d) specifying the prior for the conditional (on detection) probability of type 1 (delta_1), type 2 (delta_2), and both type 1 and type 2 encounters (1-delta_1-delta_2). If \code{a0delta} is a scaler, then this value is used for all a0delta[j] for j = 1, ..., d. For \code{mod.delta=~type}, d=3 with [delta_1, delta_2, 1-delta_1-delta_2] ~ Dirichlet(a0delta) prior. For \code{mod.delta=~1}, k=2 with [tau] ~ Beta(a0delta[1],a0delta[2]) prior, where (delta_1,delta_2,1-delta_1-delta_2) = (tau/2,tau/2,1-tau). If See McClintock et al. (2013) for more details.
-#' @param a0alpha Specifies "shape1" parameter for [alpha] ~ Beta(a0alpha, b0alpha) prior. Only applicable when \code{data.type = "sometimes"}. Default is \code{a0alpha = 1}. Note that when \code{a0alpha = 1} and \code{b0alpha = 1}, then [alpha] ~ Unif(0,1) .
-#' @param b0alpha Specifies "shape2" parameter for [alpha] ~ Beta(a0alpha, b0alpha) prior. Only applicable when \code{data.type = "sometimes"}. Default is \code{b0alpha = 1}. Note that when \code{a0alpha = 1} and \code{b0alpha = 1}, then [alpha] ~ Unif(0,1) .
-#' @param a Scale parameter for [sigma_z] ~ half-Cauchy(a) prior for the individual hetegeneity term sigma_zp = sqrt(sigma2_zp). Default is ``uninformative'' \code{a = 25} .
-#' @param mu0 Scaler or vector (of length k) specifying mean of pbeta[j] ~ Normal(mu0[j], sigma2_mu0[j]) prior. If \code{mu0} is a scaler, then this value is used for all j = 1, ..., k. Default is \code{mu0 = 0} .
-#' @param sigma2_mu0 Scaler or vector (of length k) specifying variance of pbeta[j] ~ Normal(mu0[j], sigma2_mu0[j]) prior. If \code{sigma2_mu0} is a scaler, then this value is used for all j = 1, ..., k. Default is \code{sigma2_mu0 = 1.75} .
-#' @param initial.values Optional list of \code{nchain} list(s) specifying intial values for parameters and latent variables. Default is \code{initial.values = NULL}, which causes initial values to be generated automatically. In addition to the parameters ("\code{pbeta}", "\code{N}", "\code{delta_1}", "\code{delta_2}", "\code{alpha}", "\code{sigma2_zp}", "\code{zp}", and "\code{psi}"), initial values can be specified for the initial latent history frequencies ("\code{x}") and initial individual encounter history indices ("\code{H}") .
+#' @param a0alpha Specifies "shape1" parameter for [alpha] ~ Beta(a0alpha, b0alpha) prior. Only applicable when \code{data.type = "sometimes"}. Default is \code{a0alpha = 1}. Note that when \code{a0alpha = 1} and \code{b0alpha = 1}, then [alpha] ~ Unif(0,1).
+#' @param b0alpha Specifies "shape2" parameter for [alpha] ~ Beta(a0alpha, b0alpha) prior. Only applicable when \code{data.type = "sometimes"}. Default is \code{b0alpha = 1}. Note that when \code{a0alpha = 1} and \code{b0alpha = 1}, then [alpha] ~ Unif(0,1).
+#' @param a Scale parameter for [sigma_z] ~ half-Cauchy(a) prior for the individual hetegeneity term sigma_zp = sqrt(sigma2_zp). Default is ``uninformative'' \code{a = 25}.
+#' @param mu0 Scaler or vector (of length k) specifying mean of pbeta[j] ~ Normal(mu0[j], sigma2_mu0[j]) prior. If \code{mu0} is a scaler, then this value is used for all j = 1, ..., k. Default is \code{mu0 = 0}.
+#' @param sigma2_mu0 Scaler or vector (of length k) specifying variance of pbeta[j] ~ Normal(mu0[j], sigma2_mu0[j]) prior. If \code{sigma2_mu0} is a scaler, then this value is used for all j = 1, ..., k. Default is \code{sigma2_mu0 = 1.75}.
+#' @param initial.values Optional list of \code{nchain} list(s) specifying intial values for parameters and latent variables. Default is \code{initial.values = NULL}, which causes initial values to be generated automatically. In addition to the parameters ("\code{pbeta}", "\code{N}", "\code{delta_1}", "\code{delta_2}", "\code{alpha}", "\code{sigma2_zp}", "\code{zp}", and "\code{psi}"), initial values can be specified for the initial latent history frequencies ("\code{x}") and initial individual encounter history indices ("\code{H}").
 #' @param known Optional integer vector indicating whether the encounter history of an individual is known with certainty (i.e., the observed encounter history is the true encounter history). Encounter histories with at least one type 4 encounter are automatically assumed to be known, and \code{known} does not need to be specified unless there exist encounter histories that do not contain a type 4 encounter that happen to be known with certainty (e.g., from independent telemetry studies). If specified, \code{known = c(v_1,v_2,...,v_M)} must be a vector of length \code{M = nrow(Enc.Mat)} where \code{v_i = 1} if the encounter history for individual \code{i} is known (\code{v_i = 0} otherwise). Note that known all-zero encounter histories (e.g., `000') are ignored.
-#' @param printlog Logical indicating whether to print the progress of chain(s) and any errors to a log file in the working directory. Updates are printed as 1\% increments of \code{iter} of each chain are completed. Setting \code{printlog=TRUE} is probably most useful for Windows users because progress and errors are automatically printed to the R console for "Unix-based"" machines (i.e., Mac and Linux) when \code{printlog=FALSE}. Default is \code{printlog=FALSE} .
+#' @param printlog Logical indicating whether to print the progress of chains and any errors to a log file in the working directory. Ignored when \code{nchains=1}. Updates are printed to log file as 1\% increments of \code{iter} of each chain are completed. With >1 chains, setting \code{printlog=TRUE} is probably most useful for Windows users because progress and errors are automatically printed to the R console for "Unix-like" machines (i.e., Mac and Linux) when \code{printlog=FALSE}. Default is \code{printlog=FALSE}.
 #' @param ... Additional "\code{parameters}" arguments for specifying \code{mod.p}. See \code{\link[RMark]{make.design.data}}.
 #'
 #' @details The first time \code{multimarkClosed} (or \code{\link{multimarkCJS}}) is called, it will likely produce a firewall warning alerting users that R has requested the ability to accept incoming network connections. Incoming network connections are required to use parallel processing as implemented in \code{multimarkClosed}. Note that setting \code{parms="all"} is required for any \code{multimarkClosed} model output to be used in \code{\link{multimodelClosed}}.
@@ -476,9 +477,10 @@ processClosedchains<-function(chains,params,DM,M,noccas,nchains,iter,burnin,thin
 #' test<-multimarkClosed(Enc.Mat=bobcat,data.type="never",iter=10,burnin=0,bin=5)}
 #' \donttest{
 #' # This example is excluded from testing to reduce package check time
+#' # Example uses unrealistically low values for nchain, iter, and burnin
 #' 
 #' #Run two parallel chains using the default model for bobcat data
-#' bobcat.dot<-multimarkClosed(bobcat,nchains=2)
+#' bobcat.dot<-multimarkClosed(bobcat)
 #' 
 #' #Posterior summary for monitored parameters
 #' summary(bobcat.dot$mcmc)
@@ -500,7 +502,7 @@ multimarkClosed<-function(Enc.Mat,data.type="never",covs=data.frame(),mms=NULL,m
   }
   
   parmlist<-c("pbeta","delta","N","sigma2_zp","zp","alpha","psi","H","logPosterior")
-  params <- checkClosed(parms,parmlist,mms,DM,iter,bin,thin,burnin,taccept,tuneadjust,npoints,maxnumbasis,a0delta,a0alpha,b0alpha,a,sigma2_mu0)
+  params <- checkClosed(parms,parmlist,mms,DM,iter,adapt,bin,thin,burnin,taccept,tuneadjust,npoints,maxnumbasis,a0delta,a0alpha,b0alpha,a,sigma2_mu0)
   
   data.type<-mms@data.type
   Enc.Mat<-mms@Enc.Mat
@@ -532,24 +534,21 @@ multimarkClosed<-function(Enc.Mat,data.type="never",covs=data.frame(),mms=NULL,m
   
   Prop.sd <- c(propzp,proppbeta,propsigmap)
   
-  tasks <- vector("list",nchains)
+  message("Updating...",ifelse(printlog | nchains==1,"","set 'printlog=TRUE' to follow progress of chains in a working directory log file"),"\n",sep="")
+  if(printlog & nchains==1) printlog<-FALSE
   
-  if(nchains>detectCores()) warning("Number of parallel chains (nchains) is greater than number of cores \n")
-  taskexpr <- paste0("tasks[[",1:nchains,"]]","<-function() mcmcClosed(",1:nchains,",mms,DM,params,inits,iter,adapt,bin,thin,burnin,taccept,tuneadjust,Prop.sd,gq,maxnumbasis,a0delta,a0alpha,b0alpha,a,mu0,sigma2_mu0,printlog)")
-  eval(parse(text=taskexpr))
-  names(tasks) <- paste("job", 1:length(tasks), sep = "")
-  
-  message("Updating...",ifelse(printlog,"","set 'printlog=TRUE' to follow progress of chains(s) in a working directory log file"),"\n",sep="")
-  
-  cl <- makeCluster( length(tasks) ,outfile=ifelse(printlog,paste0("multimark_log_",format(Sys.time(), "%Y-%b-%d_%H%M.%S"),".txt"),""), methods=FALSE)
-  clusterExport(cl, list("mcmcClosed","mms","DM","params","inits","iter","adapt","bin","thin","burnin","taccept","tuneadjust","Prop.sd","gq","maxnumbasis","a0delta","a0alpha","b0alpha","a","mu0","sigma2_mu0","printlog"),envir=environment())
-  chains <- clusterApply( 
-    cl,
-    tasks,
-    function(f) f()
-  )
-  stopCluster(cl)
-  gc()
+  if(nchains>1){
+    if(nchains>detectCores()) warning("Number of parallel chains (nchains) is greater than number of cores \n")
+    cl <- makeCluster( nchains ,outfile=ifelse(printlog,paste0("multimarkClosed_log_",format(Sys.time(), "%Y-%b-%d_%H%M.%S"),".txt"),""))
+    clusterExport(cl,list("mcmcClosed"),envir=environment())  
+    chains <- parLapply(cl,1:nchains, function(ichain) mcmcClosed(ichain,mms,DM,params,inits,iter,adapt,bin,thin,burnin,taccept,tuneadjust,Prop.sd,gq,maxnumbasis,a0delta,a0alpha,b0alpha,a,mu0,sigma2_mu0,printlog))
+    stopCluster(cl)
+    gc()
+  } else {
+    chains <- vector('list',nchains)
+    chains[[nchains]] <- mcmcClosed(nchains,mms,DM,params,inits,iter,adapt,bin,thin,burnin,taccept,tuneadjust,Prop.sd,gq,maxnumbasis,a0delta,a0alpha,b0alpha,a,mu0,sigma2_mu0,printlog)
+    gc()
+  }
   
   chains <- processClosedchains(chains,params,DM,M,noccas,nchains,iter,burnin,thin)
   return(list(mcmc=chains$chains,mod.p=mod.p,mod.delta=mod.delta,DM=list(p=DM$p,c=DM$c),initial.values=chains$initial.values,priorparms=priorparms))
@@ -557,14 +556,14 @@ multimarkClosed<-function(Enc.Mat,data.type="never",covs=data.frame(),mms=NULL,m
 
 #' Calculate posterior capture and recapture probabilities
 #'
-#' This function calculates posterior capture (p) and recapture (c) probabilities for each sampling occasion from \code{\link{multimarkClosed}} output. 
+#' This function calculates posterior capture (\eqn{p}) and recapture (\eqn{c}) probabilities for each sampling occasion from \code{\link{multimarkClosed}} output. 
 #'
 #'
 #' @param out List of output returned by \code{\link{multimarkClosed}}.
 #' @param link Link function for detection probability. Must be "\code{logit}" or "\code{probit}". Note that \code{\link{multimarkClosed}} is currently implemented for the logit link only.
 #' @return An object of class \code{\link[coda]{mcmc.list}} containing the following:
-#' \item{p}{Posterior samples for capture probability (p) for each sampling occasion.}
-#' \item{c}{Posterior samples for recapture probability (c) for each sampling occasion.}
+#' \item{p}{Posterior samples for capture probability (\eqn{p}) for each sampling occasion.}
+#' \item{c}{Posterior samples for recapture probability (\eqn{c}) for each sampling occasion.}
 #' @author Brett T. McClintock
 #' @seealso \code{\link{multimarkClosed}}
 #' @examples
@@ -572,9 +571,10 @@ multimarkClosed<-function(Enc.Mat,data.type="never",covs=data.frame(),mms=NULL,m
 #' test<-getprobsClosed(multimarkClosed(Enc.Mat=bobcat,data.type="never",iter=10,burnin=0,bin=5))}
 #' \donttest{
 #' # This example is excluded from testing to reduce package check time
+#' # Example uses unrealistically low values for nchain, iter, and burnin
 #' 
 #' #Run behavior model for bobcat data with constant detection probability (i.e., mod.p=~c)
-#' bobcat.c <- multimarkClosed(bobcat,mod.p=~c,nchains=2)
+#' bobcat.c <- multimarkClosed(bobcat,mod.p=~c)
 #'   
 #' #Calculate capture and recapture probabilities
 #' pc <- getprobsClosed(bobcat.c)
@@ -761,6 +761,79 @@ missingparmnamesClosed<-function(params,M,noccas,zppropsd){
   list(commonparms=commonparms,missingparms=missingparms,missingpbetaparms=missingpbetaparms,missingdeltaparms=missingdeltaparms,missingsigpparms=missingsigpparms,missingzpparms=missingzpparms,zppropsd=zppropsd,usesigp=usesigp) 
 }
 
+rjmcmcClosed <- function(ichain,M,noccas,data_type,alpha,C,All.hists,modlist,DMlist,deltalist,priorlist,mod.p.h,iter,miter,modprior,M1,monitorparms,missing,pbetapropsd,sigppropshape,sigppropscale,pmodnames,deltamodnames,gq,printlog){
+  
+  multimodel <- matrix(0,nrow=miter,ncol=length(monitorparms$parms)+1,dimnames=list(NULL,c(monitorparms$parms,"M")))
+  
+  nmod <- length(modlist)
+  mod.prob.brob <- as.brob(numeric(nmod))
+  
+  commonparms <- monitorparms$commonparms
+  
+  M.cur<- M1
+  
+  modmissingparms <- drawmissingClosed(M.cur,missing,pbetapropsd,sigppropshape,sigppropscale)
+  cur.parms <- c(modlist[[M.cur]][sample(iter,1),],modmissingparms)
+  
+  DM <- DMlist[[M.cur]]
+  DM$mod.delta <- deltalist[[M.cur]]
+  DM$mod.p.h <- mod.p.h[[M.cur]]
+  
+  cur.parms.list <- getcurClosedparmslist(cur.parms,DM,M,noccas,data_type,alpha)  
+  
+  for(iiter in 1:miter){
+    
+    mod.prob.brob[M.cur] <- getbrobprobClosed(M.cur,modprior,cur.parms["logPosterior"],cur.parms,missing,pbetapropsd,sigppropshape,sigppropscale)
+    
+    for(imod in (1:nmod)[-M.cur]){ 
+      
+      DM <- DMlist[[imod]]
+      DM$mod.delta <- deltalist[[imod]]
+      DM$mod.p.h <- mod.p.h[[imod]]
+      
+      cur.parms.list[[1]]$pbeta <- cur.parms[paste0("pbeta[",colnames(DM$p),"]")]
+      
+      loglike <- loglikeClosed(cur.parms.list[[1]],DM,noccas,C,All.hists,gq[[imod]])
+      
+      posterior <- loglike + priorsClosed(cur.parms.list[[1]],DM,priorlist[[imod]],data_type)
+      
+      mod.prob.brob[imod] <- getbrobprobClosed(imod,modprior,posterior,cur.parms,missing,pbetapropsd,sigppropshape,sigppropscale)
+    }
+    
+    if(any(is.na(as.numeric(mod.prob.brob)))){
+      warning(paste0("'NA' posterior for model '","p(",pmodnames[is.na(as.numeric(mod.prob.brob))],")delta(",deltamodnames[is.na(as.numeric(mod.prob.brob))],")' at iteration ",iiter,"; model move rejected."))
+      flush.console()
+    } else {       
+      mod.prob <- as.numeric(mod.prob.brob/Brobdingnag::sum(mod.prob.brob))
+      M.cur <- (1:nmod)[rmultinom(1, 1, mod.prob)==1]
+    }
+    
+    modmissingparms <- drawmissingClosed(M.cur,missing,pbetapropsd,sigppropshape,sigppropscale)
+    cur.parms <- c(modlist[[M.cur]][sample(iter,1),],modmissingparms)
+    
+    multimodel[iiter,"M"] <- M.cur
+    multimodel[iiter,commonparms] <- cur.parms[commonparms]
+    
+    DM <- DMlist[[M.cur]]
+    DM$mod.delta <- deltalist[[M.cur]]
+    DM$mod.p.h <- mod.p.h[[M.cur]]
+    
+    cur.parms.list <- getcurClosedparmslist(cur.parms,DM,M,noccas,data_type,alpha)  
+    
+    multimodel[iiter,monitorparms$namesp] <- monitorparms$getlogitp(DM$mod.p.h,DM$p,cur.parms.list[[1]]$pbeta,cur.parms.list[[1]]$sigma2_zp)
+    multimodel[iiter,monitorparms$namesc] <- monitorparms$getlogitc(DM$mod.p.h,DM$c,cur.parms.list[[1]]$pbeta,cur.parms.list[[1]]$sigma2_zp)[-1]
+    
+    if(!(iiter%%(miter/ min(miter,100)))) {
+      if(printlog){
+        cat("Chain ",ichain," is ",100*(iiter/miter),"% complete \n",sep="")        
+      } else{
+        cat("\rChain ",ichain," is ",100*(iiter/miter),"% complete",sep="")
+      }
+    }
+  }
+  return(multimodel)
+}
+
 #' Multimodel inference for 'multimark' closed population abundance models
 #'
 #' This function performs Bayesian multimodel inference for a set of 'multimark' closed population abundance models using the reversible jump Markov chain Monte Carlo (RJMCMC) algorithm proposed by Barker & Link (2013).
@@ -773,9 +846,10 @@ missingparmnamesClosed<-function(params,M,noccas,zppropsd){
 #' @param miter The number of RJMCMC iterations per chain. If \code{NULL}, then the number of MCMC iterations for each individual model chain is used.
 #' @param M1 Integer vector indicating the initial model for each chain, where \code{M1_j=i} initializes the RJMCMC algorithm for chain j in the model corresponding to \code{modlist[[i]]} for i=1,...,  \code{length(modlist)}. If \code{NULL}, the algorithm for all chains is initialized in the most general model. Default is \code{M1=NULL}.
 #' @param pbetapropsd Scaler specifying the standard deviation of the Normal(0, pbetapropsd) proposal distribution for "\code{pbeta}"  parameters. Default is \code{pbetapropsd=1}. See Barker & Link (2013) for more details.
-#' @param zppropsd Scaler specifying the standard deviation of the Normal(0, zppropsd) proposal distribution for "\code{zp}"  parameters. Only applies if at least one (but not all) model(s) include individual hetergeneity in detection probability. If \code{NULL}, "\code{zppropsd=sqrt(sigma2_zp)}" is used. Default is \code{zppropsd=NULL}. See Barker & Link (2013) for more details.  
+#' @param zppropsd Scaler specifying the standard deviation of the Normal(0, zppropsd) proposal distribution for "\code{zp}"  parameters. Only applies if at least one (but not all) model(s) include individual hetergeneity in detection probability. If \code{NULL}, zppropsd = sqrt(sigma2_zp) is used. Default is \code{zppropsd=NULL}. See Barker & Link (2013) for more details.  
 #' @param sigppropshape Scaler specifying the shape parameter of the invGamma(shape = sigppropshape, scale = sigppropscale) proposal distribution for "\code{sigma_zp=sqrt(sigma2_zp)}". Only applies if at least one (but not all) model(s) include individual hetergeneity in detection probability. Default is \code{sigppropshape=6}. See Barker & Link (2013) for more details.
 #' @param sigppropscale Scaler specifying the scale parameter of the invGamma(shape = sigppropshape, scale = sigppropscale) proposal distribution for "\code{sigma_zp=sqrt(sigma2_zp)}". Only applies if at least one (but not all) model(s) include individual hetergeneity in detection probability. Default is \code{sigppropscale=4}. See Barker & Link (2013) for more details.
+#' @param printlog Logical indicating whether to print the progress of chains and any errors to a log file in the working directory. Ignored when \code{nchains=1}. Updates are printed to log file as 1\% increments of \code{iter} of each chain are completed. With >1 chains, setting \code{printlog=TRUE} is probably most useful for Windows users because progress and errors are automatically printed to the R console for "Unix-like" machines (i.e., Mac and Linux) when \code{printlog=FALSE}. Default is \code{printlog=FALSE}.
 #' @details Note that setting \code{parms="all"} is required when fitting individual \code{\link{multimarkClosed}} models to be included in \code{modlist}.
 #' @return A list containing the following:
 #' \item{rjmcmc}{Reversible jump Markov chain Monte Carlo object of class \code{\link[coda]{mcmc.list}}. Includes RJMCMC output for monitored parameters and the current model at each iteration ("\code{M}").}
@@ -792,15 +866,16 @@ missingparmnamesClosed<-function(params,M,noccas,zppropsd){
 #' }
 #' \donttest{
 #' # This example is excluded from testing to reduce package check time
+#' # Example uses unrealistically low values for nchain, iter, and burnin
 #' 
 #' #Generate object of class "multimarksetup"
 #' setup <- processdata(bobcat)
 #'  
 #' #Run two parallel chains using the default model for bobcat data. Note parms="all".
-#' bobcat.dot <- multimarkClosed(mms=setup,parms="all",nchains=2)
+#' bobcat.dot <- multimarkClosed(mms=setup,parms="all")
 #' 
 #' #Run two parallel chains for bobcat data with time effects. Note parms="all".
-#' bobcat.time <- multimarkClosed(mms=setup,mod.p=~time,parms="all",nchains=2)
+#' bobcat.time <- multimarkClosed(mms=setup,mod.p=~time,parms="all")
 #' 
 #' #Perform RJMCMC using defaults
 #' modlist <- list(mod1=bobcat.dot,mod2=bobcat.time)
@@ -811,7 +886,7 @@ missingparmnamesClosed<-function(params,M,noccas,zppropsd){
 #'  
 #' #multimodel posterior summary for abundance
 #' summary(bobcat.M$rjmcmc[,"N"])}
-multimodelClosed<-function(mms,modlist,modprior=rep(1/length(modlist),length(modlist)),monparms="N",miter=NULL,M1=NULL,pbetapropsd=1,zppropsd=NULL,sigppropshape=6,sigppropscale=4){
+multimodelClosed<-function(mms,modlist,modprior=rep(1/length(modlist),length(modlist)),monparms="N",miter=NULL,M1=NULL,pbetapropsd=1,zppropsd=NULL,sigppropshape=6,sigppropscale=4,printlog=FALSE){
   
   nmod <- length(modlist)
   iter <- unlist(unique(lapply(modlist,function(x) unique(lapply(x$mcmc,nrow)))))
@@ -826,6 +901,7 @@ multimodelClosed<-function(mms,modlist,modprior=rep(1/length(modlist),length(mod
   noccas<-ncol(mms@Enc.Mat)
   M<-nrow(mms@Enc.Mat)
   All.hists<-matrix(mms@vAll.hists,byrow=TRUE,ncol=noccas)
+  C<-mms@C
   gq <- lapply(modlist,function(x) gauss.quad(x$priorparms$npoints,kind="hermite"))
   
   if(is.null(miter)) miter <- iter
@@ -836,20 +912,17 @@ multimodelClosed<-function(mms,modlist,modprior=rep(1/length(modlist),length(mod
   checkparmsClosed(mms,modlist,params,parmlist=c("pbeta[(Intercept)]","N","psi",paste0("H[",1:M,"]"),"logPosterior"),M)
   
   message("\nPerforming closed population Bayesian multimodel inference by RJMCMC \n")
-  cat(paste0("mod",1:nmod,": ","p(",pmodnames,")delta(",deltamodnames,")"),"",sep="\n")
+  message(paste0("mod",1:nmod,": ","p(",pmodnames,")delta(",deltamodnames,")\n"))
   
   missing <- missingparmnamesClosed(params,M,noccas,zppropsd) 
   
   monitorparms <- monitorparmsClosed(monparms,c(missing$commonparms,"p","c"),noccas)
   
-  commonparms <- monitorparms$commonparms
-  
-  multimodel <- lapply(vector('list',nchains),function(x) x=matrix(0,nrow=miter,ncol=length(monitorparms$parms)+1,dimnames=list(NULL,c(monitorparms$parms,"M"))))
-  
+  DMlist <- lapply(modlist,function(x) x$DM)
+  deltalist <- lapply(modlist,function(x) x$mod.delta)
+  priorlist <- lapply(modlist,function(x) x$priorparms) 
   mod.p.h <- unlist(lapply(modlist,function(x) any("h"==attributes(terms(x$mod.p))$term.labels)))
   
-  mod.prob.brob <- as.brob(numeric(nmod))
-  
   data_type <- mms@data.type
   if(data_type=="never"){
     alpha <- 0
@@ -858,67 +931,23 @@ multimodelClosed<-function(mms,modlist,modprior=rep(1/length(modlist),length(mod
   } else {
     alpha <- numeric(0)
   }
+ 
+
+  message("Updating...",ifelse(printlog | nchains==1,"","set 'printlog=TRUE' to follow progress of chains in a working directory log file"),"\n",sep="")
+  if(printlog & nchains==1) printlog<-FALSE
   
-  pb <- txtProgressBar(min=1,max=nchains*miter+1,char="+",width=100,style=3)
-  setTxtProgressBar(pb, 1)
-  for(ichain in 1:nchains){
-    
-    M.cur<- M1[ichain]
-    
-    modmissingparms <- drawmissingClosed(M.cur,missing,pbetapropsd,sigppropshape,sigppropscale)
-    cur.parms <- c(modlist[[M.cur]]$mcmc[[ichain]][sample(iter,1),],modmissingparms)
-    
-    DM <- modlist[[M.cur]]$DM
-    DM$mod.delta <- modlist[[M.cur]]$mod.delta
-    DM$mod.p.h <- mod.p.h[[M.cur]]
-    
-    cur.parms.list <- getcurClosedparmslist(cur.parms,DM,M,noccas,data_type,alpha)  
-    
-    for(iiter in 1:miter){
-      
-      mod.prob.brob[M.cur] <- getbrobprobClosed(M.cur,modprior,cur.parms["logPosterior"],cur.parms,missing,pbetapropsd,sigppropshape,sigppropscale)
-      
-      for(imod in (1:nmod)[-M.cur]){ 
-        
-        DM <- modlist[[imod]]$DM
-        DM$mod.delta <- modlist[[imod]]$mod.delta
-        DM$mod.p.h <- mod.p.h[imod]
-        
-        cur.parms.list[[1]]$pbeta <- cur.parms[paste0("pbeta[",colnames(DM$p),"]")]
-        
-        loglike <- loglikeClosed(cur.parms.list[[1]],DM,noccas,mms@C,All.hists,gq[[imod]])
-        
-        posterior <- loglike + priorsClosed(cur.parms.list[[1]],DM,modlist[[imod]]$priorparms,data_type)
-        
-        mod.prob.brob[imod] <- getbrobprobClosed(imod,modprior,posterior,cur.parms,missing,pbetapropsd,sigppropshape,sigppropscale)
-      }
-      
-      if(any(is.na(as.numeric(mod.prob.brob)))){
-        warning(paste0("'NA' posterior for model '","p(",pmodnames[is.na(as.numeric(mod.prob.brob))],")delta(",deltamodnames[is.na(as.numeric(mod.prob.brob))],")' at iteration ",iiter,"; model move rejected."))
-      } else {       
-        mod.prob <- as.numeric(mod.prob.brob/Brobdingnag::sum(mod.prob.brob))
-        M.cur <- (1:nmod)[rmultinom(1, 1, mod.prob)==1]
-      }
-      
-      modmissingparms <- drawmissingClosed(M.cur,missing,pbetapropsd,sigppropshape,sigppropscale)
-      cur.parms <- c(modlist[[M.cur]]$mcmc[[ichain]][sample(iter,1),],modmissingparms)
-      
-      multimodel[[ichain]][iiter,"M"] <- M.cur
-      multimodel[[ichain]][iiter,commonparms] <- cur.parms[commonparms]
-      
-      DM <- modlist[[M.cur]]$DM
-      DM$mod.delta <- modlist[[M.cur]]$mod.delta
-      DM$mod.p.h <- mod.p.h[[M.cur]]
-      
-      cur.parms.list <- getcurClosedparmslist(cur.parms,DM,M,noccas,data_type,alpha) 
-      
-      multimodel[[ichain]][iiter,monitorparms$namesp] <- monitorparms$getlogitp(DM$mod.p.h,DM$p,cur.parms.list[[1]]$pbeta,cur.parms.list[[1]]$sigma2_zp)
-      multimodel[[ichain]][iiter,monitorparms$namesc] <- monitorparms$getlogitc(DM$mod.p.h,DM$c,cur.parms.list[[1]]$pbeta,cur.parms.list[[1]]$sigma2_zp)[-1]
-      
-      setTxtProgressBar(pb, (ichain-1)*miter+iiter+1)
-    }
+  if(nchains>1){
+    if(nchains>detectCores()) warning("Number of parallel chains (nchains) is greater than number of cores \n")
+    cl <- makeCluster( nchains ,outfile=ifelse(printlog,paste0("multimodelClosed_log_",format(Sys.time(), "%Y-%b-%d_%H%M.%S"),".txt"),""))
+    clusterExport(cl,list("rjmcmcClosed"),envir=environment())
+    multimodel <- parLapply(cl,1:nchains, function(ichain) rjmcmcClosed(ichain,M,noccas,data_type,alpha,C,All.hists,lapply(modlist,function(x) x$mcmc[[ichain]]),DMlist,deltalist,priorlist,mod.p.h,iter,miter,modprior,M1[ichain],monitorparms,missing,pbetapropsd,sigppropshape,sigppropscale,pmodnames,deltamodnames,gq,printlog))
+    stopCluster(cl)
+    gc()
+  } else {
+    multimodel <- vector('list',nchains)
+    multimodel[[nchains]] <- rjmcmcClosed(nchains,M,noccas,data_type,alpha,C,All.hists,lapply(modlist,function(x) x$mcmc[[nchains]]),DMlist,deltalist,priorlist,mod.p.h,iter,miter,modprior,M1,monitorparms,missing,pbetapropsd,sigppropshape,sigppropscale,pmodnames,deltamodnames,gq,printlog)
+    gc()
   }
-  close(pb)
   
   pos.prob <- vector('list',nchains)
   for(ichain in 1:nchains){
